function [dpds, dens_m, dens_ns, Hl] = dpBeggs_Brill(ql, qg, dens_l, dens_g, visc_l, visc_g, theta, step_length, ID)

global g

v_sg = qg/(pi*(ID^2)/4);
v_sl = ql/(pi*(ID^2)/4);
v_m = v_sg + v_sl;

Cl = v_sl/v_m;  % No slip liquid holdup

%% Liquid holdup correlation here:
% Beggs and Brill.
L1 = 316*Cl^0.302;
L2 = 0.0009252*Cl^(-2.4684);
L3 = 0.1*Cl^(-1.4516);
L4 = 0.5*Cl^(-6.738);
Frm = v_m^2/(g*ID);
% sigma = 0.0012;   % Assumed liquid surface tension. Correlation should be implemented instead.
% Nlv = 1.938*v_sl*(dens_l/sigma)^(1/4);   % Uncertain about the units that should be used. Nlv is almost dimensionless, but not quite.
sigma = 50; % Dynes/cm
Nlv = 1.938*v_sl/0.3048*(dens_l*0.0624279606/sigma)^(1/4);  % Converted to field units.
Hl = 1;

if (Cl<0.01 && Frm<L1) || (Cl>=0.01 && Frm<L2)
    % Segregated flow
    Hl_0 = (0.98*Cl^0.4846)/(Frm^0.0868);
    if theta>=0
        beta = (1-Cl)*log(0.011*Nlv^3.539*Cl^-3.768*Frm^-1.614);
    else
        beta = (1-Cl)*log(4.70*Nlv^0.1244*Cl^-0.3692*Frm^-0.5056);
    end
    if beta < 0
        beta =0;
    end
    
    B_theta = 1+beta*(sind(1.8*theta)-(sind(1.8*theta))^3/3);
    Hl = Hl_0 * B_theta;
elseif (Cl>=0.01 && Cl<0.4 && Frm>L3 && Frm<=L1) || (Cl>=0.4 && Frm>L3 && Frm<=L4)
    % Intermittent flow
    Hl_0 = (0.845*Cl^0.5351)/(Frm^0.0173);
    if theta>=0
        beta = (1-Cl)*log(2.96*Nlv^-0.4473*Cl^0.305*Frm^-0.0978);
    else
        beta = (1-Cl)*log(4.70*Nlv^0.1244*Cl^-0.3692*Frm^-0.5056);
    end
    if beta < 0
        beta =0;
    end
    
    B_theta = 1+beta*(sind(1.8*theta)-(sind(1.8*theta))^3/3);
    Hl = Hl_0 * B_theta;
elseif (Cl<0.4 && Frm>=L1) || (Cl>=0.4 && Frm>L4)
    % Distributed flow
    Hl_0 = (1.065*Cl^0.5824)/(Frm^0.609);

    if theta>=0
        beta = 0;
    else
        beta = (1-Cl)*log(4.70*Nlv^0.1244*Cl^-0.3692*Frm^-0.5056);
    end
    if beta < 0
        beta =0;
    end
    
    B_theta = 1+beta*(sind(1.8*theta)-(sind(1.8*theta))^3/3);
    Hl = Hl_0 * B_theta;
elseif (Cl>=0.001 && Frm>L2 && Frm<L3)
    % Transitional flow
    A = (L3 - Frm)/(L3 - L2);
    B = 1-A;

        % Segregated flow
    Hl_0_s = (0.98*Cl^0.4846)/(Frm^0.0868);
        % Intermittent flow
    Hl_0_i = (0.845*Cl^0.5351)/(Frm^0.0173);
    if theta>=0
        beta_s = (1-Cl)*log(0.011*Nlv^3.539*Cl^-3.768*Frm^-1.614);
        beta_i = (1-Cl)*log(2.96*Nlv^-0.4473*Cl^0.305*Frm^-0.0978);
    else
        beta_s = (1-Cl)*log(4.70*Nlv^0.1244*Cl^-0.3692*Frm^-0.5056);
        beta_i = beta_s;
    end
    if beta_s < 0
        beta_s =0;
    end
    if beta_i < 0
        beta_i =0;
    end
    
    B_theta_s = 1+beta_s*(sind(1.8*theta)-(sind(1.8*theta))^3/3);
    B_theta_i = 1+beta_i*(sind(1.8*theta)-(sind(1.8*theta))^3/3);
    Hl_s = Hl_0_s * B_theta_s;
    Hl_i = Hl_0_i * B_theta_i;
    
    Hl = A*Hl_s + B*Hl_i;  
end

if Hl<Cl
    Hl=Cl;
end
if Hl>1
    Hl = 1;
end


%%


dens_m = dens_l*Hl + dens_g*(1-Hl);    % Slip included (true mixture values)
%visc_m = visc_l*Hl + visc_g*(1-Hl);

dens_ns = dens_l*Cl + dens_g*(1-Cl);    % No slip
visc_ns = visc_l*Cl + visc_g*(1-Cl);

% Previously 1488
Re_ns = 1000 * (dens_ns*v_m*ID)/visc_ns;   % Viscosity in cp.
f_ns = (2*log10(Re_ns/(4.5223*log10(Re_ns)-3.8215)))^-2;   % No-slip friction factor in smooth pipe
x = Cl/(Hl^2);

if x==0
    S = 0;
elseif x>1 && x<1.2
    S = log(2.2*x-1.2);
else
    S = log(x)/(-0.0523+3.182*log(x)-0.8725*(log(x))^2+0.01853*(log(x))^4);
end

f_tp = f_ns*exp(S);   % Two phase friction factor.

dp_fric = f_tp/2 * dens_ns * v_m^2*step_length/ID;
dp_grav = g*dens_m*step_length*sind(theta);

dpds = dp_fric + dp_grav;

% Accelleration may also be included. See Thiago's paper. Here the
% dimensions/units seem wrong, though.

end